@@ -56,20 +56,6 @@ main (int   argc,
     gint retval;
     NautilusApplication *application;
 
-#if defined (HAVE_MALLOPT) && defined(M_MMAP_THRESHOLD)
-    /* Nautilus uses lots and lots of small and medium size allocations,
-     * and then a few large ones for the desktop background. By default
-     * glibc uses a dynamic treshold for how large allocations should
-     * be mmaped. Unfortunately this triggers quickly for nautilus when
-     * it does the desktop background allocations, raising the limit
-     * such that a lot of temporary large allocations end up on the
-     * heap and are thus not returned to the OS. To fix this we set
-     * a hardcoded limit. I don't know what a good value is, but 128K
-     * was the old glibc static limit, lets use that.
-     */
-    mallopt (M_MMAP_THRESHOLD, 128 * 1024);
-#endif
-
     if (g_getenv ("NAUTILUS_DEBUG") != NULL)
     {
         eel_make_warnings_and_criticals_stop_in_debugger ();
