@@ -41,6 +41,7 @@
 
 #include <eel/eel-debug.h>
 #include <eel/eel-gtk-extensions.h>
+#include <eel/eel-vfs-extensions.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <gdk/gdkx.h>
 #include <gdk/gdkkeysyms.h>
@@ -115,11 +116,11 @@ typedef struct
     GtkWidget *main_view;
 
     /* Notifications */
-    GtkWidget *notification_delete;
-    GtkWidget *notification_delete_label;
-    GtkWidget *notification_delete_close;
-    GtkWidget *notification_delete_undo;
-    guint notification_delete_timeout_id;
+    GtkWidget *in_app_notification_undo;
+    GtkWidget *in_app_notification_undo_label;
+    GtkWidget *in_app_notification_undo_close_button;
+    GtkWidget *in_app_notification_undo_undo_button;
+    guint in_app_notification_undo_timeout_id;
     GtkWidget *notification_operation;
     GtkWidget *notification_operation_label;
     GtkWidget *notification_operation_close;
@@ -174,7 +175,8 @@ static const struct
     { GDK_KEY_Forward, "forward" },
 };
 
-static const GtkPadActionEntry pad_actions[] = {
+static const GtkPadActionEntry pad_actions[] =
+{
     { GTK_PAD_ACTION_BUTTON, 0, -1, N_("Parent folder"), "up" },
     { GTK_PAD_ACTION_BUTTON, 1, -1, N_("Home"), "go-home" },
     { GTK_PAD_ACTION_BUTTON, 2, -1, N_("New tab"), "new-tab" },
@@ -789,7 +791,7 @@ nautilus_window_new_tab (NautilusWindow *window)
     NautilusWindowSlot *current_slot;
     NautilusWindowOpenFlags flags;
     GFile *location;
-    char *scheme;
+    g_autofree gchar *uri = NULL;
 
     current_slot = nautilus_window_get_active_slot (window);
     location = nautilus_window_slot_get_location (current_slot);
@@ -798,8 +800,8 @@ nautilus_window_new_tab (NautilusWindow *window)
     {
         flags = g_settings_get_enum (nautilus_preferences, NAUTILUS_PREFERENCES_NEW_TAB_POSITION);
 
-        scheme = g_file_get_uri_scheme (location);
-        if (strcmp (scheme, "x-nautilus-search") == 0)
+        uri = g_file_get_uri (location);
+        if (eel_uri_is_search (uri))
         {
             location = g_file_new_for_path (g_get_home_dir ());
         }
@@ -808,8 +810,6 @@ nautilus_window_new_tab (NautilusWindow *window)
             g_object_ref (location);
         }
 
-        g_free (scheme);
-
         flags |= NAUTILUS_WINDOW_OPEN_FLAG_NEW_TAB;
         nautilus_window_open_location_full (window, location, flags, NULL, NULL);
         g_object_unref (location);
@@ -1127,12 +1127,12 @@ places_sidebar_show_other_locations_with_flags (NautilusWindow     *window,
 }
 
 static void
-places_sidebar_show_starred_location (NautilusWindow    *window,
-                                      GtkPlacesOpenFlags open_flags)
+places_sidebar_show_starred_location (NautilusWindow     *window,
+                                      GtkPlacesOpenFlags  open_flags)
 {
     GFile *location;
 
-    location = g_file_new_for_uri ("favorites:///");
+    location = g_file_new_for_uri ("starred:///");
 
     open_location_cb (window, location, open_flags);
 
@@ -1216,7 +1216,7 @@ places_sidebar_drag_action_requested_cb (GtkPlacesSidebar *sidebar,
     }
     uri = g_file_get_uri (dest_file);
 
-    if (g_list_length (items) < 1)
+    if (items == NULL)
     {
         goto out;
     }
@@ -1701,17 +1701,17 @@ remove_notifications (NautilusWindow *window)
 
     priv = nautilus_window_get_instance_private (window);
     /* Hide it inmediatily so we can animate the new notification. */
-    transition_type = gtk_revealer_get_transition_type (GTK_REVEALER (priv->notification_delete));
-    gtk_revealer_set_transition_type (GTK_REVEALER (priv->notification_delete),
+    transition_type = gtk_revealer_get_transition_type (GTK_REVEALER (priv->in_app_notification_undo));
+    gtk_revealer_set_transition_type (GTK_REVEALER (priv->in_app_notification_undo),
                                       GTK_REVEALER_TRANSITION_TYPE_NONE);
-    gtk_revealer_set_reveal_child (GTK_REVEALER (priv->notification_delete),
+    gtk_revealer_set_reveal_child (GTK_REVEALER (priv->in_app_notification_undo),
                                    FALSE);
-    gtk_revealer_set_transition_type (GTK_REVEALER (priv->notification_delete),
+    gtk_revealer_set_transition_type (GTK_REVEALER (priv->in_app_notification_undo),
                                       transition_type);
-    if (priv->notification_delete_timeout_id != 0)
+    if (priv->in_app_notification_undo_timeout_id != 0)
     {
-        g_source_remove (priv->notification_delete_timeout_id);
-        priv->notification_delete_timeout_id = 0;
+        g_source_remove (priv->in_app_notification_undo_timeout_id);
+        priv->in_app_notification_undo_timeout_id = 0;
     }
 
     transition_type = gtk_revealer_get_transition_type (GTK_REVEALER (priv->notification_operation));
@@ -1729,53 +1729,54 @@ remove_notifications (NautilusWindow *window)
 }
 
 static void
-hide_notification_delete (NautilusWindow *window)
+hide_in_app_notification_undo (NautilusWindow *window)
 {
     NautilusWindowPrivate *priv;
 
     priv = nautilus_window_get_instance_private (window);
 
-    if (priv->notification_delete_timeout_id != 0)
+    if (priv->in_app_notification_undo_timeout_id != 0)
     {
-        g_source_remove (priv->notification_delete_timeout_id);
-        priv->notification_delete_timeout_id = 0;
+        g_source_remove (priv->in_app_notification_undo_timeout_id);
+        priv->in_app_notification_undo_timeout_id = 0;
     }
 
-    gtk_revealer_set_reveal_child (GTK_REVEALER (priv->notification_delete), FALSE);
+    gtk_revealer_set_reveal_child (GTK_REVEALER (priv->in_app_notification_undo), FALSE);
 }
 
 static void
-nautilus_window_on_notification_delete_undo_clicked (GtkWidget      *notification,
-                                                     NautilusWindow *window)
+on_in_app_notification_undo_undo_button_clicked (GtkWidget      *notification,
+                                                 NautilusWindow *window)
 {
-    hide_notification_delete (window);
+    hide_in_app_notification_undo (window);
 
     nautilus_file_undo_manager_undo (GTK_WINDOW (window));
 }
 
 static void
-nautilus_window_on_notification_delete_close_clicked (GtkWidget      *notification,
-                                                      NautilusWindow *window)
+on_in_app_notification_undo_close_button_clicked (GtkWidget      *notification,
+                                                  NautilusWindow *window)
 {
-    hide_notification_delete (window);
+    hide_in_app_notification_undo (window);
 }
 
 static gboolean
-nautilus_window_on_notification_delete_timeout (NautilusWindow *window)
+on_in_app_notification_undo_timeout (NautilusWindow *window)
 {
-    hide_notification_delete (window);
+    hide_in_app_notification_undo (window);
 
-    return FALSE;
+    return G_SOURCE_REMOVE;
 }
 
-static char *
-nautilus_window_notification_delete_get_label (NautilusFileUndoInfo *undo_info,
-                                               GList                *files)
+static gchar *
+in_app_notification_undo_deleted_get_label (NautilusFileUndoInfo *undo_info)
 {
+    GList *files;
     gchar *file_label;
     gchar *label;
     gint length;
 
+    files = nautilus_file_undo_info_trash_get_files (NAUTILUS_FILE_UNDO_INFO_TRASH (undo_info));
     length = g_list_length (files);
     if (length == 1)
     {
@@ -1794,6 +1795,33 @@ nautilus_window_notification_delete_get_label (NautilusFileUndoInfo *undo_info,
     return label;
 }
 
+static gchar *
+in_app_notification_undo_unstar_get_label (NautilusFileUndoInfo *undo_info)
+{
+    GList *files;
+    gchar *label;
+    gint length;
+
+    files = nautilus_file_undo_info_starred_get_files (NAUTILUS_FILE_UNDO_INFO_STARRED (undo_info));
+    length = g_list_length (files);
+    if (length == 1)
+    {
+        g_autofree gchar *file_label = NULL;
+
+        file_label = nautilus_file_get_display_name (files->data);
+        /* Translators: one item has been unstarred and %s is its name. */
+        label = g_markup_printf_escaped (_("“%s” unstarred"), file_label);
+    }
+    else
+    {
+        /* Translators: one or more items have been unstarred, and %d
+         * is the count. */
+        label = g_markup_printf_escaped (ngettext ("%d file unstarred", "%d files unstarred", length), length);
+    }
+
+    return label;
+}
+
 static void
 nautilus_window_on_undo_changed (NautilusFileUndoManager *manager,
                                  NautilusWindow          *window)
@@ -1801,37 +1829,66 @@ nautilus_window_on_undo_changed (NautilusFileUndoManager *manager,
     NautilusWindowPrivate *priv;
     NautilusFileUndoInfo *undo_info;
     NautilusFileUndoManagerState state;
-    gchar *label;
-    GList *files;
 
     priv = nautilus_window_get_instance_private (window);
     undo_info = nautilus_file_undo_manager_get_action ();
     state = nautilus_file_undo_manager_get_state ();
 
     if (undo_info != NULL &&
-        state == NAUTILUS_FILE_UNDO_MANAGER_STATE_UNDO &&
-        nautilus_file_undo_info_get_op_type (undo_info) == NAUTILUS_FILE_UNDO_OP_MOVE_TO_TRASH)
+        state == NAUTILUS_FILE_UNDO_MANAGER_STATE_UNDO)
     {
-        files = nautilus_file_undo_info_trash_get_files (NAUTILUS_FILE_UNDO_INFO_TRASH (undo_info));
+        gboolean popup_notification = FALSE;
+        g_autofree gchar *label = NULL;
 
-        /* Don't pop up a notification if user canceled the operation or the focus
-         * is not in the this window. This is an easy way to know from which window
-         * was the delete operation made */
-        if (g_list_length (files) > 0 && gtk_window_has_toplevel_focus (GTK_WINDOW (window)))
+        if (nautilus_file_undo_info_get_op_type (undo_info) == NAUTILUS_FILE_UNDO_OP_MOVE_TO_TRASH)
         {
-            label = nautilus_window_notification_delete_get_label (undo_info, files);
-            gtk_label_set_markup (GTK_LABEL (priv->notification_delete_label), label);
-            gtk_revealer_set_reveal_child (GTK_REVEALER (priv->notification_delete), TRUE);
-            priv->notification_delete_timeout_id = g_timeout_add_seconds (NOTIFICATION_TIMEOUT,
-                                                                                  (GSourceFunc) nautilus_window_on_notification_delete_timeout,
-                                                                                  window);
-            g_free (label);
+            g_autoptr (GList) files = NULL;
+
+            files = nautilus_file_undo_info_trash_get_files (NAUTILUS_FILE_UNDO_INFO_TRASH (undo_info));
+
+            /* Don't pop up a notification if user canceled the operation or the focus
+             * is not in the this window. This is an easy way to know from which window
+             * was the delete operation made */
+            if (files != NULL && gtk_window_has_toplevel_focus (GTK_WINDOW (window)))
+            {
+                popup_notification = TRUE;
+                label = in_app_notification_undo_deleted_get_label (undo_info);
+            }
+        }
+        else if (nautilus_file_undo_info_get_op_type (undo_info) == NAUTILUS_FILE_UNDO_OP_STARRED)
+        {
+            NautilusWindowSlot *active_slot;
+            GFile *location;
+
+            active_slot = nautilus_window_get_active_slot (window);
+            location = nautilus_window_slot_get_location (active_slot);
+            /* Don't pop up a notification if the focus is not in the this
+             * window. This is an easy way to know from which window was the
+             * unstart operation made */
+            if (eel_uri_is_starred (g_file_get_uri (location)) &&
+                gtk_window_has_toplevel_focus (GTK_WINDOW (window)) &&
+                !nautilus_file_undo_info_starred_is_starred (NAUTILUS_FILE_UNDO_INFO_STARRED (undo_info)))
+            {
+                popup_notification = TRUE;
+                label = in_app_notification_undo_unstar_get_label (undo_info);
+            }
+        }
+
+        if (popup_notification)
+        {
+            remove_notifications (window);
+            gtk_label_set_markup (GTK_LABEL (priv->in_app_notification_undo_label),
+                                  label);
+            gtk_revealer_set_reveal_child (GTK_REVEALER (priv->in_app_notification_undo),
+                                           TRUE);
+            priv->in_app_notification_undo_timeout_id = g_timeout_add_seconds (NOTIFICATION_TIMEOUT,
+                                                                               (GSourceFunc) on_in_app_notification_undo_timeout,
+                                                                               window);
         }
-        g_list_free (files);
     }
     else
     {
-        hide_notification_delete (window);
+        hide_in_app_notification_undo (window);
     }
 }
 
@@ -1919,8 +1976,8 @@ nautilus_window_show_operation_notification (NautilusWindow *window,
 
         gtk_revealer_set_reveal_child (GTK_REVEALER (priv->notification_operation), TRUE);
         priv->notification_operation_timeout_id = g_timeout_add_seconds (NOTIFICATION_TIMEOUT,
-                                                                                 (GSourceFunc) on_notification_operation_timeout,
-                                                                                 window);
+                                                                         (GSourceFunc) on_notification_operation_timeout,
+                                                                         window);
     }
 }
 
@@ -2034,7 +2091,7 @@ notebook_popup_menu_show (NautilusWindow *window,
     gtk_widget_show_all (popup);
 
     gtk_menu_popup_at_pointer (GTK_MENU (popup),
-                               (GdkEvent*) event);
+                               (GdkEvent *) event);
 }
 
 /* emitted when the user clicks the "close" button of tabs */
@@ -2053,7 +2110,7 @@ notebook_button_press_cb (GtkWidget      *widget,
 {
     NautilusWindow *window = user_data;
 
-    if (GDK_BUTTON_PRESS == event->type && 3 == event->button)
+    if (GDK_BUTTON_PRESS == event->type && event->button == GDK_BUTTON_SECONDARY)
     {
         notebook_popup_menu_show (window, event);
         return TRUE;
@@ -2472,10 +2529,10 @@ nautilus_window_finalize (GObject *object)
         priv->sidebar_width_handler_id = 0;
     }
 
-    if (priv->notification_delete_timeout_id != 0)
+    if (priv->in_app_notification_undo_timeout_id != 0)
     {
-        g_source_remove (priv->notification_delete_timeout_id);
-        priv->notification_delete_timeout_id = 0;
+        g_source_remove (priv->in_app_notification_undo_timeout_id);
+        priv->in_app_notification_undo_timeout_id = 0;
     }
 
     if (priv->notification_operation_timeout_id != 0)
@@ -2815,10 +2872,10 @@ nautilus_window_init (NautilusWindow *window)
     g_type_ensure (NAUTILUS_TYPE_NOTEBOOK);
     gtk_widget_init_template (GTK_WIDGET (window));
 
-    g_signal_connect_object (priv->notification_delete_close, "clicked",
-                             G_CALLBACK (nautilus_window_on_notification_delete_close_clicked), window, 0);
-    g_signal_connect_object (priv->notification_delete_undo, "clicked",
-                             G_CALLBACK (nautilus_window_on_notification_delete_undo_clicked), window, 0);
+    g_signal_connect_object (priv->in_app_notification_undo_close_button, "clicked",
+                             G_CALLBACK (on_in_app_notification_undo_close_button_clicked), window, 0);
+    g_signal_connect_object (priv->in_app_notification_undo_undo_button, "clicked",
+                             G_CALLBACK (on_in_app_notification_undo_undo_button_clicked), window, 0);
 
     priv->slots = NULL;
     priv->active_slot = NULL;
@@ -2830,7 +2887,7 @@ nautilus_window_init (NautilusWindow *window)
     gtk_window_group_add_window (window_group, GTK_WINDOW (window));
     g_object_unref (window_group);
 
-    priv->tab_data_queue = g_queue_new();
+    priv->tab_data_queue = g_queue_new ();
 
     priv->pad_controller = gtk_pad_controller_new (GTK_WINDOW (window),
                                                    G_ACTION_GROUP (window),
@@ -2878,10 +2935,10 @@ nautilus_window_class_init (NautilusWindowClass *class)
     gtk_widget_class_bind_template_child_private (wclass, NautilusWindow, places_sidebar);
     gtk_widget_class_bind_template_child_private (wclass, NautilusWindow, main_view);
     gtk_widget_class_bind_template_child_private (wclass, NautilusWindow, notebook);
-    gtk_widget_class_bind_template_child_private (wclass, NautilusWindow, notification_delete);
-    gtk_widget_class_bind_template_child_private (wclass, NautilusWindow, notification_delete_label);
-    gtk_widget_class_bind_template_child_private (wclass, NautilusWindow, notification_delete_undo);
-    gtk_widget_class_bind_template_child_private (wclass, NautilusWindow, notification_delete_close);
+    gtk_widget_class_bind_template_child_private (wclass, NautilusWindow, in_app_notification_undo);
+    gtk_widget_class_bind_template_child_private (wclass, NautilusWindow, in_app_notification_undo_label);
+    gtk_widget_class_bind_template_child_private (wclass, NautilusWindow, in_app_notification_undo_undo_button);
+    gtk_widget_class_bind_template_child_private (wclass, NautilusWindow, in_app_notification_undo_close_button);
     gtk_widget_class_bind_template_child_private (wclass, NautilusWindow, notification_operation);
     gtk_widget_class_bind_template_child_private (wclass, NautilusWindow, notification_operation_label);
     gtk_widget_class_bind_template_child_private (wclass, NautilusWindow, notification_operation_open);
@@ -2948,7 +3005,7 @@ nautilus_event_get_window_open_flags (void)
     }
 
     if ((event->type == GDK_BUTTON_PRESS || event->type == GDK_BUTTON_RELEASE) &&
-        (event->button.button == 2))
+        (event->button.button == GDK_BUTTON_MIDDLE))
     {
         flags |= NAUTILUS_WINDOW_OPEN_FLAG_NEW_TAB;
     }
